"""FastAPI reproduction of the MCPwn twin-asymmetry shape.

The /mcp decorator takes a Depends(auth); /mcp_message doesn't.
"""
from fastapi import Depends, FastAPI

app = FastAPI()


def auth() -> None:
    return None


@app.post("/mcp", dependencies=[Depends(auth)])
async def mcp_main() -> dict:
    return {"ok": True}


@app.post("/mcp_message")
async def mcp_message() -> dict:
    # Missing Depends(auth) — MCPwn shape.
    return {"ok": True}
