import logging

logger = logging.getLogger(__name__)


def handle_auth(bearer_token: str) -> None:
    logger.info("got auth header (redacted): Bearer ***")
    logger.info("session_token=<redacted>")
