// Express reproduction. `authMw` is only applied to /mcp; /mcp_message
// is wide open — MCPwn shape.

import express from "express";

const app = express();
const authMw = (_req: any, _res: any, next: any) => next();
const handler = (_req: any, res: any) => res.json({ ok: true });

app.post("/mcp", authMw, handler);
app.post("/mcp_message", handler);
