"""Second variant: os.system over data read from stdin (Ox family)."""
import json
import os
import sys
from mcp.server.fastmcp import FastMCP  # noqa: F401


def handle() -> None:
    payload = json.loads(sys.stdin.read())
    os.system(payload["command"])
