import logging

logger = logging.getLogger(__name__)


def handle_auth(bearer_token: str) -> None:
    logger.info("got auth header: Bearer %s", bearer_token)
    logger.info(f"session_token={bearer_token}")
